/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
*  Copyright 2013 - 2020, nymea GmbH
*  Contact: contact@nymea.io
*
*  This file is part of nymea.
*  This project including source code and documentation is protected by copyright law, and
*  remains the property of nymea GmbH. All rights, including reproduction, publication,
*  editing and translation, are reserved. The use of this project is subject to the terms of a
*  license agreement to be concluded with nymea GmbH in accordance with the terms
*  of use of nymea GmbH, available under https://nymea.io/license
*
*  GNU General Public License Usage
*  Alternatively, this project may be redistributed and/or modified under
*  the terms of the GNU General Public License as published by the Free Software Foundation,
*  GNU version 3. this project is distributed in the hope that it will be useful, but WITHOUT ANY
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
*  PURPOSE. See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this project.
*  If not, see <https://www.gnu.org/licenses/>.
*
*  For any further details and any questions please contact us under contact@nymea.io
*  or see our FAQ/Licensing Information on https://nymea.io/license/faq
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef AUTHENTICATIONPROCESS_H
#define AUTHENTICATIONPROCESS_H

#include <QObject>
#include <QProcess>
#include <QElapsedTimer>
#include <QNetworkAccessManager>

#include "userinformation.h"
#include "authentication/authenticator.h"

namespace remoteproxy {

class AuthenticationProcess : public QObject
{
    Q_OBJECT
public:
    explicit AuthenticationProcess(QNetworkAccessManager *manager, const QString &accessKey, const QString &secretAccessKey, const QString &sessionToken, QObject *parent = nullptr);

private:
    QNetworkAccessManager *m_manager = nullptr;
    QString m_accessKey;
    QString m_secretAccessKey;
    QString m_sessionToken;

    QString m_token;
    QString m_resultFileName;

    bool m_fallback = false;

    QProcess *m_process = nullptr;
    QElapsedTimer m_lambdaTimer;
    QElapsedTimer m_processTimer;

    void invokeLambdaFunction();
    void startVerificationProcess();

    void cleanUp();

signals:
    void authenticationFinished(Authenticator::AuthenticationError error, const UserInformation &userInformation = UserInformation());

private slots:
    void onLambdaInvokeFunctionFinished();
    void onProcessFinished(int exitCode, QProcess::ExitStatus exitStatus);

public slots:
    void authenticate(const QString &token);

};

}

#endif // AUTHENTICATIONPROCESS_H
